import os
import pathlib
import unittest

import badkeys

TDPATH = f"{os.path.dirname(__file__)}/data/"


class TestSmallprimes(unittest.TestCase):
    def test_smallprimes_valid(self):
        key = pathlib.Path(f"{TDPATH}rsa-smallfactors-valid.key").read_text()
        r = badkeys.checkpubkey(key, checks=["smallfactors"])
        self.assertTrue("smallfactors" in r["results"])
        self.assertEqual(r["results"]["smallfactors"]["subtest"], "valid")

    def test_smallprimes_corrupt(self):
        key = pathlib.Path(f"{TDPATH}rsa-smallfactors-corrupt.key").read_text()
        r = badkeys.checkpubkey(key, checks=["smallfactors"])
        self.assertTrue("smallfactors" in r["results"])
        self.assertEqual(r["results"]["smallfactors"]["subtest"], "corrupt")

    def test_smallprimes_nofinding(self):
        key = pathlib.Path(f"{TDPATH}rsa-ok.key").read_text()
        r = badkeys.checkpubkey(key, checks=["smallfactors"])
        self.assertFalse(r["results"])


if __name__ == "__main__":
    unittest.main()



import json
import os
import pathlib
import unittest

import badkeys

TDPATH = f"{os.path.dirname(__file__)}/data/"


class TestJwk(unittest.TestCase):

    @unittest.skipUnless(os.environ.get("RUN_ONLINETESTS"), "Skipping blocklist tests")
    def test_jwkbl(self):
        jkey = json.loads(pathlib.Path(f"{TDPATH}jwk-ecp256-rfc7517.json").read_text())
        r = badkeys.checkjwk(jkey, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["type"] == "ec")
        self.assertTrue(r["curve"] == "p256")

        jkey = json.loads(pathlib.Path(f"{TDPATH}jwk-rsa-rfc7517.json").read_text())
        r = badkeys.checkjwk(jkey, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["type"] == "rsa")
        self.assertTrue(r["bits"] == 2048)

        jkey = json.loads(pathlib.Path(f"{TDPATH}jwk-ed25519-rfc8037.json").read_text())
        r = badkeys.checkjwk(jkey, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["type"] == "ec")
        self.assertTrue(r["curve"] == "ed25519")

    def test_jwk(self):
        jkey = json.loads(pathlib.Path(f"{TDPATH}jwk-rsa-ok.json").read_text())
        r = badkeys.checkjwk(jkey, checks=["roca", "pattern", "sharedprimes", "fermat"])
        self.assertFalse(r["results"])
        self.assertTrue(r["type"] == "rsa")
        self.assertTrue(r["bits"] == 2048)

        jkey = json.loads(pathlib.Path(f"{TDPATH}jwk-curvename-invalid.json").read_text())
        r = badkeys.checkjwk(jkey, checks=["roca", "pattern", "sharedprimes", "fermat"])
        self.assertTrue(r["type"] == "unparseable")


if __name__ == "__main__":
    unittest.main()



import os
import unittest

import badkeys

TDPATH = f"{os.path.dirname(__file__)}/data/"


class TestSharedprimes(unittest.TestCase):
    def test_sharedprimes(self):
        with open(f"{TDPATH}rsa-sharedprimes.key") as f:
            key = f.read()
        r = badkeys.checkpubkey(key, checks=["sharedprimes"])
        self.assertTrue("sharedprimes" in r["results"])
        self.assertTrue(r["bits"] == 2048)
        with open(f"{TDPATH}rsa-ok.key") as f:
            key = f.read()
        r = badkeys.checkpubkey(key, checks=["sharedprimes"])
        self.assertFalse(r["results"])
        self.assertTrue(r["bits"] == 2048)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: MIT
# (c) Hanno Böck
#
# Part of badkeys: https://badkeys.info/

import os
import pathlib
import unittest

import badkeys

TDPATH = os.path.join(os.path.dirname(__file__), "data")


class TestSmalld(unittest.TestCase):

    def test_smalld(self):

        # Use all checks except blocklist to make sure they have no
        # issues with large d RSA values
        mychecks = list(badkeys.allchecks.keys())
        mychecks.remove("blocklist")

        # key with mixed-up d/e values (d=65537)
        key = pathlib.Path(f"{TDPATH}/rsa-small-d.key").read_text()
        r = badkeys.detectandcheck(key, checks=mychecks)
        self.assertTrue("smalld" in r["results"])
        self.assertTrue("rsawarnings" in r["results"])
        self.assertTrue(r["results"]["rsawarnings"]["subtest"] == "exponent_not_65537")

        # key with large e, but otherwise correct (FIPS test vector)
        key = pathlib.Path(f"{TDPATH}/rsa-large-e.key").read_text()
        r = badkeys.detectandcheck(key, checks=mychecks)
        self.assertFalse("smalld" in r["results"])
        self.assertTrue("rsawarnings" in r["results"])
        self.assertTrue(r["results"]["rsawarnings"]["subtest"] == "exponent_not_65537")


if __name__ == "__main__":
    unittest.main()



import os
import pathlib
import unittest

import badkeys

TDPATH = f"{os.path.dirname(__file__)}/data"


class TestDnssec(unittest.TestCase):

    @unittest.skipUnless(os.environ.get("RUN_ONLINETESTS"), "Skipping blocklist tests")
    def test_dnssecbl(self):
        key = pathlib.Path(f"{TDPATH}/dnssec/dnssec-p256-rfc6605.dnskey").read_text()
        r = badkeys.checkdnskey(key, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["type"] == "ec")
        self.assertTrue(r["curve"] == "p256")

        key = pathlib.Path(f"{TDPATH}/dnssec/dnssec-root-rsa.dnskey").read_text()
        r = badkeys.checkdnskey(key, checks=["blocklist"])
        self.assertFalse("blocklist" in r["results"])
        self.assertTrue(r["type"] == "rsa")
        self.assertTrue(r["bits"] == 2048)

        key = pathlib.Path(f"{TDPATH}/dnssec/dnssec-rsa-large-e.dnskey").read_text()
        r = badkeys.checkdnskey(key, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["type"] == "rsa")
        self.assertTrue(r["bits"] == 4096)
        self.assertTrue(r["e"] == (1 << 3064 | 1))

    def test_dnssec(self):
        key = pathlib.Path(f"{TDPATH}/dnssec/dnssec-p256-rfc6605.dnskey").read_text()
        r = badkeys.checkdnskey(key, checks=["roca", "pattern", "sharedprimes", "fermat"])
        self.assertFalse(r["results"])
        self.assertTrue(r["type"] == "ec")
        self.assertTrue(r["curve"] == "p256")

        key = pathlib.Path(f"{TDPATH}/dnssec/dnssec-root-rsa.dnskey").read_text()
        r = badkeys.checkdnskey(key, checks=["roca", "pattern", "sharedprimes", "fermat"])
        self.assertFalse(r["results"])
        self.assertTrue(r["type"] == "rsa")
        self.assertTrue(r["bits"] == 2048)
        self.assertTrue(r["e"] == 65537)

        key = pathlib.Path(f"{TDPATH}/dnssec/dnssec-rsa-large-e.dnskey").read_text()
        r = badkeys.checkdnskey(key, checks=["roca", "pattern", "sharedprimes", "fermat"])
        self.assertFalse(r["results"])
        self.assertTrue(r["type"] == "rsa")
        self.assertTrue(r["bits"] == 4096)
        self.assertTrue(r["e"] == (1 << 3064 | 1))


if __name__ == "__main__":
    unittest.main()



# Tests for bugs in past versions of badkeys

import os
import pathlib
import unittest

import badkeys

TDPATH = f"{os.path.dirname(__file__)}/data/"


class TestRegressions(unittest.TestCase):
    def test_rsanzero(self):
        # Regression test for https://github.com/badkeys/badkeys/issues/31
        key = pathlib.Path(f"{TDPATH}rsazerozero.key").read_text()
        r = badkeys.checkpubkey(key, checks=["xzbackdoor"])
        self.assertFalse(r["results"])
        key = pathlib.Path(f"{TDPATH}rsanzero.key").read_text()
        r = badkeys.checkpubkey(key, checks=["xzbackdoor"])
        self.assertFalse(r["results"])


if __name__ == "__main__":
    unittest.main()



import os
import unittest

import badkeys

TDPATH = f"{os.path.dirname(__file__)}/data/"


class TestBlocklist(unittest.TestCase):

    @unittest.skipUnless(os.environ.get("RUN_ONLINETESTS"), "Skipping blocklist tests")
    def test_rsabl(self):
        with open(f"{TDPATH}rsa-debianweak.key") as f:
            key = f.read()
        r = badkeys.checkpubkey(key, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["bits"] == 2048)
        with open(f"{TDPATH}rsa-ok.key") as f:
            key = f.read()
        r = badkeys.checkpubkey(key, checks=["blocklist"])
        self.assertFalse(r["results"])
        self.assertTrue(r["bits"] == 2048)

    @unittest.skipUnless(os.environ.get("RUN_ONLINETESTS"), "Skipping blocklist tests")
    def test_ecbl(self):
        with open(f"{TDPATH}ec-p256-rfc-example.key") as f:
            key = f.read()
        r = badkeys.checkpubkey(key, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["curve"] == "p256")
        with open(f"{TDPATH}ed25519-rfc-example.key") as f:
            key = f.read()
        r = badkeys.checkpubkey(key, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["curve"] == "ed25519")
        with open(f"{TDPATH}x448-ok.key") as f:
            key = f.read()
        r = badkeys.checkpubkey(key, checks=["blocklist"])
        self.assertFalse(r["results"])
        self.assertTrue(r["curve"] == "x448")

    # Testing key in SSH DSA pubkey format.
    # Python cryptography plans to deprecate this format,
    # we will need to find a solution.
    @unittest.skipUnless(os.environ.get("RUN_ONLINETESTS"), "Skipping blocklist tests")
    def test_dsabl(self):
        with open(f"{TDPATH}dsa-sshpub-ietf-example.key") as f:
            key = f.read()
        r = badkeys.checksshpubkey(key, checks=["blocklist"])
        self.assertTrue("blocklist" in r["results"])
        self.assertTrue(r["bits"] == 1024)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: MIT
# (c) Hanno Böck

import os
import pathlib
import unittest

from cryptography.hazmat.primitives.asymmetric import ed25519, rsa

from badkeys.checks import _checkkey, allchecks
from badkeys.dkim import parsedkim

TDPATH = f"{os.path.dirname(__file__)}/data/"


class TestDkim(unittest.TestCase):

    def test_dkimparser(self):
        # Defect inputs should not return anything, but also should not cause
        # unexpected exceptions
        for dkimbroken in ["dkim-broken-char.txt", "dkim-broken-length.txt"]:
            dkey = pathlib.Path(f"{TDPATH}dkim/{dkimbroken}").read_text(errors="ignore")
            self.assertTrue(isinstance(parsedkim(dkey), str))
        # Valid inputs should return PEM public key
        for dkimvalid in ["dkim-valid-gmail.txt", "dkim-insecure-rfc8463.txt",
                          "dkim-escaped-quote.txt", "dkim-comment-quotes.txt"]:
            dkey = pathlib.Path(f"{TDPATH}dkim/{dkimvalid}").read_text()
            parsedkey = parsedkim(dkey)
            self.assertTrue(isinstance(parsedkey, (ed25519.Ed25519PublicKey, rsa.RSAPublicKey)))

    @unittest.skipUnless(os.environ.get("RUN_ONLINETESTS"), "Skipping blocklist tests")
    def test_dkimscan(self):
        dkey = pathlib.Path(f"{TDPATH}dkim/dkim-valid-gmail.txt").read_text()
        ret = _checkkey(parsedkim(dkey), checks=allchecks)
        self.assertFalse(ret["results"])

        for dkimbl in ["dkim-insecure-rfc8463.txt", "dkim-escaped-quote.txt",
                       "dkim-comment-quotes.txt"]:
            dkey = pathlib.Path(f"{TDPATH}dkim/{dkimbl}").read_text()
            ret = _checkkey(parsedkim(dkey), checks=allchecks)
            self.assertTrue("blocklist" in ret["results"])


if __name__ == "__main__":
    unittest.main()



import os
import pathlib
import unittest

import badkeys

TDPATH = f"{os.path.dirname(__file__)}/data/"


class TestFailures(unittest.TestCase):
    def test_brokencrt(self):
        brokencrt = "-----BEGIN CERTIFICATE-----\nMII"
        r = badkeys.detectandcheck(brokencrt)
        self.assertEqual(r["type"], "unparseable")

    def test_brokencsr(self):
        brokencsr = "-----BEGIN CERTIFICATE REQUEST-----\nMII"
        r = badkeys.detectandcheck(brokencsr)
        self.assertEqual(r["type"], "unparseable")

    def test_empty(self):
        r = badkeys.detectandcheck("")
        self.assertEqual(r["type"], "notfound")

    def test_unsupportedssh(self):
        r = badkeys.detectandcheck("ssh-invalid invalid")
        self.assertEqual(r["type"], "unsupported")

    def test_defect(self):
        indir = os.path.join(TDPATH, "defect")
        for fn in os.listdir(indir):
            indata = pathlib.Path(os.path.join(indir, fn)).read_text()
            r = badkeys.detectandcheck(indata)
            self.assertEqual(r["type"], "unparseable")

    def test_unsupported(self):
        indir = os.path.join(TDPATH, "unsupported")
        for fn in os.listdir(indir):
            indata = pathlib.Path(os.path.join(indir, fn)).read_text()
            r = badkeys.detectandcheck(indata)
            # some "unsupported" inputs return "unparseable" due to
            # cryptography's API behavior
            self.assertIn(r["type"], ["unparseable", "unsupported"])


if __name__ == "__main__":
    unittest.main()
